<template>
    <div>
        <Head title="User"/>
        <v-card dense>
            <v-data-table :headers="headers" :items="users" class="elevation-1">
                <template v-slot:top>

                    <v-dialog v-model="dialogEdit" max-width="500px">
                        <v-card>
                            <v-toolbar color="primary" dark>Edit Member</v-toolbar>
                            <v-card-text>
                                <v-container>
                                    <v-row>
                                        <v-col cols="6">
                                            <v-text-field v-model="editedItem.name" label="Name" :rules="rules"
                                                          :error-messages="error  && !editedItem.name ? ['This field is required.'] : ''"></v-text-field>
                                        </v-col>
                                        <v-col cols="6">
                                            <v-text-field v-model="editedItem.email" label="Email" :rules="rules"
                                                          :error-messages="error  && !editedItem.email ? ['This field is required.'] : ''"></v-text-field>
                                        </v-col>
                                        <v-col cols="12">
                                            <v-text-field label="photo photo" v-model="editedItem.photo"
                                                          placeholder="Paste URL or click Browse">
                                                <v-btn @click="selectPhoto" text slot="append" dense x-small
                                                       v-if="!editedItem.photo">
                                                    Browse
                                                    <v-icon x-small right color="primary">mdi-folder-open</v-icon>
                                                </v-btn>
                                                <v-btn @click="editedItem.photo = ''" text slot="append" dense x-small
                                                       v-else>
                                                    Remove
                                                    <v-icon x-small right color="error">mdi-delete</v-icon>
                                                </v-btn>
                                            </v-text-field>
                                        </v-col>

                                    </v-row>
                                </v-container>
                            </v-card-text>
                            <v-card-actions>
                                <v-spacer></v-spacer>
                                <v-btn color="primary" text @click="close"> Cancel</v-btn>
                                <v-btn color="primary" text @click="save">Save</v-btn>
                            </v-card-actions>
                        </v-card>
                    </v-dialog>

                </template>
                <template v-slot:item.role="{ item }">
                    <v-select :ref="`roleSelection_${item.id}`" @change="updateRole(item.id, $event)" rounded dense
                              outlined :value="item.role_id" :items="roles" item-text="name" item-value="id"
                              hide-details>
                        <template v-slot:item="scope">
                            {{ scope.item.name }}
                            <v-spacer/>
                            <v-icon right v-if="item.role && item.role.id === scope.item.id">mdi-check</v-icon>
                        </template>
                    </v-select>
                </template>
                <template v-slot:item.name="{ item }">
                    <v-row justify="space-between">
                        <v-col cols="auto">
                            <v-avatar size="42px">
                                <v-img :src="item.photo"/>
                            </v-avatar>
                        </v-col>
                        <v-col>
                            <div class="subtitle-2">{{ item.name }}</div>
                            <div class="caption">{{ item.email }}</div>
                        </v-col>
                    </v-row>
                </template>
                <template v-slot:item.active="{ item }">
                    <v-icon dense v-if="item.active" color="success">mdi-check</v-icon>
                    <v-icon dense v-else color="warning">mdi-cancel</v-icon>
                </template>
                <template v-slot:item.actions="{ item }">
                    <v-menu bottom left dense offset-y>
                        <template v-slot:activator="{ on, attrs }">
                            <v-btn :disabled="item.id === $page.props.auth.user.id" small icon v-bind="attrs" v-on="on">
                                <v-icon>mdi-dots-vertical</v-icon>
                            </v-btn>
                        </template>
                        <v-list dense class="py-0">
                            <v-list-item @click="updateActive(item)" link>
                                <v-list-item-title>
                                    <v-icon small v-if="item.active">mdi-account-cancel-outline</v-icon>
                                    <v-icon small v-else>mdi-account-check-outline</v-icon>
                                    <span v-if="item.active">Deactivate User</span>
                                    <span v-else>Activate User</span>
                                </v-list-item-title>
                            </v-list-item>
                            <v-list-item @click="editItem(item)" link>
                                <v-list-item-title>
                                    <v-icon small>mdi-pencil</v-icon>
                                    Edit User
                                </v-list-item-title>
                            </v-list-item>
                            <v-list-item @click="resetPassword(item)" link>
                                <v-list-item-title>
                                    <v-icon small>mdi-account-key-outline</v-icon>
                                    Reset Password
                                </v-list-item-title>
                            </v-list-item>
                            <v-divider/>
                            <v-list-item @click="deleteItem(item)" link class="error--text">
                                <v-list-item-title>
                                    <v-icon small color="error">mdi-account-minus-outline</v-icon>
                                    Delete User
                                </v-list-item-title>
                            </v-list-item>
                        </v-list>
                    </v-menu>
                </template>
            </v-data-table>
        </v-card>
    </div>
</template>

<script>
import SettingLayout from "../../../Components/Layouts/Settings.vue";
import Setting from "./../Setting.vue";
import User from "./User.vue";

export default {
    name: "UserIndex",
    layout: [SettingLayout, Setting, User],
    data() {
        return {
            error: false,
            rules: [
                value => !!value || 'Please fill this field.'
            ],
            dialogEdit: false,
            headers: [
                {text: 'Member', align: 'start', value: 'name'},
                /*{text: 'Email', value: 'email'},*/
                {text: 'Role', value: 'role', align: 'center', width: '12px'},
                {text: 'Active', value: 'active'},
                {text: 'Action', value: 'actions', sortable: false},
            ],
            users: this.$page.props.users,
            roles: this.$page.props.roles,
            editedIndex: -1,
            editedItem: {
                name: '',
                email: '',
                photo: {}
            },
            defaultItem: {
                name: '',
                email: '',
                photo: {}
            },
        }

    },

    computed: {
        formTitle() {
            return this.editedIndex === -1 ? 'Updated member' : 'Edited member'
        },
    },

    watch: {
        dialog(val) {
            val || this.close()
        },
        dialogDelete(val) {
            val || this.closeDelete()
        },
    },

    methods: {
        selectPhoto() {
            let eventName = 'landlord-profilexxx'
            this.eventSubscription = this.$store.subscribe((mutation, state) => {
                if (mutation.type === "Media/SET_EVENT_NAME" && mutation.payload === eventName) {
                    if (state.Media.media.length) {
                        this.form.photo = state.Media.media[0]
                        this.eventSubscription()
                    }
                }
            });
            this.showMediaDialog({event: eventName, type: 'image'})
        },

        updateRole(user, role) {
            this._confirmDialog({
                message: "Are you sure you want to update this user's role?",
                confirm: () => {
                    axios.put(this.$route('role.assign', {user, role}))
                        .then(resp => resp.data)
                        .then(data => {
                            this.showSnack(['User role updated successfully.', 'success'])
                        })
                        .catch(error => {
                            // undo selection
                            this.$refs[`roleSelection_${user}`].setValue(this.users.find(u => u.id === user).role_id)
                            this.validationErrors(error.response.data.errors, {}, error.response.status)
                            this.showSnack(["Permission wasn't updated. Please, try again.", "error"])
                        })
                }
            })
        },

        updateActive(item) {
            this._confirmDialog({
                message: "Are you sure you want to do this?",
                confirm: () => {
                    axios.put(this.$route('user.set_active', item.id))
                        .then(resp => resp.data)
                        .then(data => {
                            this.users.splice(this.users.indexOf(item), 1, data.data)
                            this.showSnack({message: `Action successful.`})
                        })
                        .catch(err => {
                            console.log(err.response)
                        })
                }
            })
        },

        resetPassword(item) {
            this._confirmDialog({
                title: `Reset ${item.name}'s password`,
                message: `<div class="v-input v-input--is-label-active v-input--is-dirty theme--light v-text-field v-text-field--is-booted v-text-field--placeholder">
                                    <div class="v-input__control">
                                    <div class="v-input__slot">
                                    <div class="v-text-field__slot">
                                    <input onclick="this.select();" id="newPassword" placeholder="Enter new password" type="text">
                                    </div>
                                    </div></div></div>`,
                confirmText: 'Reset Password',
                type: 'warning',
                onMount: () => {
                    setTimeout(() => {
                        let elt = document.getElementById('newPassword')
                        elt.focus()
                    }, 500)
                },
                confirm: () => {
                    let password = document.getElementById('newPassword').value
                    if (password) {
                        axios.put(this.$route('user.reset_password', {user: item.id}), {password})
                            .then(resp => resp.data)
                            .then(data => {

                            })
                    } else {
                        this.showSnack(['Password is empty.', 'warning'])
                    }
                }
            })
        },

        editItem(item) {
            this.editedIndex = this.users.indexOf(item)
            this.editedItem = Object.assign({}, item)
            this.dialogEdit = true
        },

        deleteItem(item) {
            this._confirmDialog({
                message: 'Are you sure you want to delete this member?',
                type: 'error',
                cancelColor: 'success',
                confirm: () => {
                    this.editedIndex = this.users.indexOf(item)
                    this.editedItem = Object.assign({}, item)
                    this.users.splice(this.editedIndex, 1)
                    this.closeDelete()
                    this.$inertia.delete(this.$route('user.delete', this.editedItem.id), {
                        onBefore: () => {
                        },
                        onSuccess: (page) => {
                            this.showSnack(["Successfully deleted member!"]);
                        },
                        onError: (errors) => {
                            console.log(errors)
                            this.showSnack(["Member not deleted!", "error"]);
                        },
                        onFinish: visit => {
                        },
                    });
                }
            })
        },

        close() {
            this.dialogEdit = false
            this.$nextTick(() => {
                this.editedItem = Object.assign({}, this.defaultItem)
                this.editedIndex = -1
            })
        },

        closeDelete() {
            this.$nextTick(() => {
                this.editedItem = Object.assign({}, this.defaultItem)
                this.editedIndex = -1
            })
        },

        save() {
            if (this.editedIndex > -1) {
                this.$inertia.put(this.$route('user.update', this.users[this.editedIndex].id), this.editedItem, {
                    onBefore: () => {
                    },
                    onSuccess: (page) => {
                        this.users = page.props.users
                        this.showSnack(["Successfully updated member!"])
                    },
                    onError: (errors) => {
                        this.error = true
                        this.showSnack(["Member not updated!", "error"])
                    },
                });
                this.close()
            }
        },
    },
    mounted() {
    }
}
</script>

<style>

</style>
